@@ -3,7 +3,7 @@
 /* ========================================================================= */
 
 /* ------------------------------------------------------------------------- */
-/* AMD Version 1.2, Copyright (c) 2005 by Timothy A. Davis,		     */
+/* AMD Version 2.0, Copyright (c) 2006 by Timothy A. Davis,		     */
 /* Patrick R. Amestoy, and Iain S. Duff.  See ../README.txt for License.     */
 /* email: davis at cise.ufl.edu    CISE Department, Univ. of Florida.        */
 /* web: http://www.cise.ufl.edu/research/sparse/amd                          */
@@ -13,7 +13,7 @@
  * factorization of P*A*P' has fewer nonzeros and takes less work than the
  * Cholesky factorization of A.  If A is not symmetric, then it performs its
  * ordering on the matrix A+A'.  Two sets of user-callable routines are
- * provided, one for "int" integers and the other for "long" integers.
+ * provided, one for int integers and the other for UF_long integers.
  *
  * The method is based on the approximate minimum degree algorithm, discussed
  * in Amestoy, Davis, and Duff, "An approximate degree ordering algorithm",
@@ -44,7 +44,12 @@ extern "C" {
 /* get the definition of size_t: */
 #include <stddef.h>
 
-int amd_order (		    /* returns 0 if OK, negative value if error */
+/* define UF_long */
+#include "UFconfig.h"
+
+int amd_order		    /* returns AMD_OK, AMD_OK_BUT_JUMBLED,
+			     * AMD_INVALID, or AMD_OUT_OF_MEMORY */
+(
     int n,		    /* A is n-by-n.  n must be >= 0. */
     const int Ap [ ],	    /* column pointers for A, of size n+1 */
     const int Ai [ ],	    /* row indices of A, of size nz = Ap [n] */
@@ -53,11 +58,12 @@ int amd_order (		    /* returns 0 if OK, negative value if error */
     double Info [ ]	    /* output Info statistics, of size AMD_INFO */
 ) ;
 
-long amd_l_order (	    /* see above for description of arguments */
-    long n,
-    const long Ap [ ],
-    const long Ai [ ],
-    long P [ ],
+UF_long amd_l_order	    /* see above for description of arguments */
+(
+    UF_long n,
+    const UF_long Ap [ ],
+    const UF_long Ai [ ],
+    UF_long P [ ],
     double Control [ ],
     double Info [ ]
 ) ;
@@ -65,15 +71,15 @@ long amd_l_order (	    /* see above for description of arguments */
 /* Input arguments (not modified):
  *
  *	n: the matrix A is n-by-n.
- *	Ap: an int/long array of size n+1, containing the column pointers of A.
- *	Ai: an int/long array of size nz, containing the row indices of A,
+ *	Ap: an int/UF_long array of size n+1, containing column pointers of A.
+ *	Ai: an int/UF_long array of size nz, containing the row indices of A,
  *	    where nz = Ap [n].
  *	Control:  a double array of size AMD_CONTROL, containing control
  *	    parameters.  Defaults are used if Control is NULL.
  *
  * Output arguments (not defined on input):
  *
- *	P: an int/long array of size n, containing the output permutation. If
+ *	P: an int/UF_long array of size n, containing the output permutation. If
  *	    row i is the kth pivot row, then P [k] = i.  In MATLAB notation,
  *	    the reordered matrix is A (P,P).
  *	Info: a double array of size AMD_INFO, containing statistical
@@ -81,8 +87,15 @@ long amd_l_order (	    /* see above for description of arguments */
  *
  * On input, the matrix A is stored in column-oriented form.  The row indices
  * of nonzero entries in column j are stored in Ai [Ap [j] ... Ap [j+1]-1].
- * The row indices must appear in ascending order in each column, and there
- * must not be any duplicate entries.  Row indices must be in the range 0 to
+ *
+ * If the row indices appear in ascending order in each column, and there
+ * are no duplicate entries, then amd_order is slightly more efficient in
+ * terms of time and memory usage.  If this condition does not hold, a copy
+ * of the matrix is created (where these conditions do hold), and the copy is
+ * ordered.  This feature is new to v2.0 (v1.2 and earlier required this
+ * condition to hold for the input matrix).
+ * 
+ * Row indices must be in the range 0 to
  * n-1.  Ap [0] must be zero, and thus nz = Ap [n] is the number of nonzeros
  * in A.  The array Ap is of size n+1, and the array Ai is of size nz = Ap [n].
  * The matrix does not need to be symmetric, and the diagonal does not need to
@@ -92,17 +105,12 @@ long amd_l_order (	    /* see above for description of arguments */
  * pattern of the matrix A is the same as that used internally by MATLAB.
  * If you wish to use a more flexible input structure, please see the
  * umfpack_*_triplet_to_col routines in the UMFPACK package, at
- * http://www.cise.ufl.edu/research/sparse/umfpack, or use the amd_preprocess
- * routine discussed below.
+ * http://www.cise.ufl.edu/research/sparse/umfpack.
  *
  * Restrictions:  n >= 0.  Ap [0] = 0.  Ap [j] <= Ap [j+1] for all j in the
- *	range 0 to n-1.  nz = Ap [n] >= 0.  For all j in the range 0 to n-1,
- *	and for all p in the range Ap [j] to Ap [j+1]-2, Ai [p] < Ai [p+1] must
- *	hold.  Ai [0..nz-1] must be in the range 0 to n-1.  To avoid integer
- *	overflow, (2.4*nz + 8*n) < INT_MAX / sizeof (int) for must hold for the
- *	"int" version. (2.4*nz + 8*n) < LONG_MAX / sizeof (long) must hold
- *	for the "long" version.  Finally, Ai, Ap, and P must not be NULL.  If
- *	any of these restrictions are not met, AMD returns AMD_INVALID.
+ *	range 0 to n-1.  nz = Ap [n] >= 0.  Ai [0..nz-1] must be in the range 0
+ *	to n-1.  Finally, Ai, Ap, and P must not be NULL.  If any of these
+ *	restrictions are not met, AMD returns AMD_INVALID.
  *
  * AMD returns:
  *
@@ -114,6 +122,9 @@ long amd_l_order (	    /* see above for description of arguments */
  *	AMD_INVALID if the input arguments n, Ap, Ai are invalid, or if P is
  *	    NULL.
  *
+ *	AMD_OK_BUT_JUMBLED if the matrix had unsorted columns, and/or duplicate
+ *	    entries, but was otherwise valid.
+ *
  * The AMD routine first forms the pattern of the matrix A+A', and then
  * computes a fill-reducing ordering, P.  If P [k] = i, then row/column i of
  * the original is the kth pivotal row.  In MATLAB notation, the permuted
@@ -152,7 +163,7 @@ long amd_l_order (	    /* see above for description of arguments */
  * condition.
  * 
  *	Info [AMD_STATUS]:  the return value of AMD, either AMD_OK,
- *	    AMD_OUT_OF_MEMORY, or AMD_INVALID.
+ *	    AMD_OK_BUT_JUMBLED, AMD_OUT_OF_MEMORY, or AMD_INVALID.
  *
  *	Info [AMD_N]: n, the size of the input matrix
  *
@@ -213,121 +224,20 @@ long amd_l_order (	    /* see above for description of arguments */
  *	    future versions.
  */    
 
-/* ------------------------------------------------------------------------- */
-/* AMD preprocess */
-/* ------------------------------------------------------------------------- */
-
-/* amd_preprocess: sorts, removes duplicate entries, and transposes the
- * nonzero pattern of a column-form matrix A, to obtain the matrix R.
- *
- * Alternatively, you can consider this routine as constructing a row-form
- * matrix from a column-form matrix.  Duplicate entries are allowed in A (and
- * removed in R). The columns of R are sorted.  Checks its input A for errors.
- *
- * On input, A can have unsorted columns, and can have duplicate entries.
- * Ap [0] must still be zero, and Ap must be monotonically nondecreasing.
- * Row indices must be in the range 0 to n-1.
- *
- * On output, if this routine returns AMD_OK, then the matrix R is a valid
- * input matrix for AMD_order.  It has sorted columns, with no duplicate
- * entries in each column.  Since AMD_order operates on the matrix A+A', it
- * can just as easily use A or A', so the transpose has no significant effect
- * (except for minor tie-breaking, which can lead to a minor effect in the
- * quality of the ordering).  As an example, compare the output of amd_demo.c
- * and amd_demo2.c.
- *
- * This routine transposes A to get R because that's the simplest way to
- * sort and remove duplicate entries from a matrix.
- *
- * Allocates 2*n integer work arrays, and free's them when done.
- *
- * If you wish to call amd_order, but do not know if your matrix has unsorted
- * columns or duplicate entries, then you can use the following code, which is
- * fairly efficient.  amd_order will not allocate any internal matrix until
- * it checks that the input matrix is valid, so the method below is memory-
- * efficient as well.  This code snippet assumes that Rp and Ri are already
- * allocated, and are the same size as Ap and Ai respectively.
-
-    result = amd_order (n, p, Ap, Ai, Control, Info) ;
-    if (result == AMD_INVALID)
-    {
-	if (amd_preprocess (n, Ap, Ai, Rp, Ri) == AMD_OK)
-	{
-	    result = amd_order (n, p, Rp, Ri, Control, Info) ;
-	}
-    }
-
- * amd_preprocess will still return AMD_INVALID if any row index in Ai is out
- * of range or if the Ap array is invalid.  These errors are not corrected by
- * amd_preprocess since they represent a more serious error that should be
- * flagged with the AMD_INVALID error code.
- *
- * You may also call amd_valid directly (see below).
- */ 
-
-int amd_preprocess
-(
-    int n,
-    const int Ap [ ],
-    const int Ai [ ],
-    int Rp [ ],
-    int Ri [ ]
-) ;
-
-long amd_l_preprocess
-(
-    long n,
-    const long Ap [ ],
-    const long Ai [ ],
-    long Rp [ ],
-    long Ri [ ]
-) ;
-
-/* Input arguments (not modified):
- *
- *	n: the matrix A is n-by-n.
- *	Ap: an int/long array of size n+1, containing the column pointers of A.
- *	Ai: an int/long array of size nz, containing the row indices of A,
- *	    where nz = Ap [n].
- *	The nonzero pattern of column j of A is in Ai [Ap [j] ... Ap [j+1]-1].
- *	Ap [0] must be zero, and Ap [j] <= Ap [j+1] must hold for all j in the
- *	range 0 to n-1.  Row indices in Ai must be in the range 0 to n-1.
- *	The row indices in any one column need not be sorted, and duplicates
- *	may exist.
- *
- * Output arguments (not defined on input):
- *
- *	Rp: an int/long array of size n+1, containing the column pointers of R.
- *	Ri: an int/long array of size rnz, containing the row indices of R,
- *	    where rnz = Rp [n].  Note that Rp [n] will be less than Ap [n] if
- *	    duplicates appear in A.  In general, Rp [n] <= Ap [n].
- *      The data structure for R is the same as A, except that each column of
- *      R contains sorted row indices, and no duplicates appear in any column.
- *
- * amd_preprocess returns:
- *
- *	AMD_OK if the matrix A is valid and sufficient memory can be allocated
- *	    to perform the preprocessing.
- *
- *	AMD_OUT_OF_MEMORY if not enough memory can be allocated.
- *
- *	AMD_INVALID if the input arguments n, Ap, Ai are invalid, or if Rp or
- *	    Ri are NULL.
- */
-
 /* ------------------------------------------------------------------------- */
 /* direct interface to AMD */
 /* ------------------------------------------------------------------------- */
 
-/* This is the primary AMD ordering routine.  It is not meant to be
+/* amd_2 is the primary AMD ordering routine.  It is not meant to be
  * user-callable because of its restrictive inputs and because it destroys
  * the user's input matrix.  It does not check its inputs for errors, either.
  * However, if you can work with these restrictions it can be faster than
  * amd_order and use less memory (assuming that you can create your own copy
  * of the matrix for AMD to destroy).  Refer to AMD/Source/amd_2.c for a
- * description. */
+ * description of each parameter. */
 
-void amd_2 (
+void amd_2
+(
     int n,
     int Pe [ ],
     int Iw [ ],
@@ -345,20 +255,21 @@ void amd_2 (
     double Info [ ]
 ) ;
 
-void amd_l2 (
-    long n,
-    long Pe [ ],
-    long Iw [ ],
-    long Len [ ],
-    long iwlen,
-    long pfree,
-    long Nv [ ],
-    long Next [ ], 
-    long Last [ ],
-    long Head [ ],
-    long Elen [ ],
-    long Degree [ ],
-    long W [ ],
+void amd_l2
+(
+    UF_long n,
+    UF_long Pe [ ],
+    UF_long Iw [ ],
+    UF_long Len [ ],
+    UF_long iwlen,
+    UF_long pfree,
+    UF_long Nv [ ],
+    UF_long Next [ ], 
+    UF_long Last [ ],
+    UF_long Head [ ],
+    UF_long Elen [ ],
+    UF_long Degree [ ],
+    UF_long W [ ],
     double Control [ ],
     double Info [ ]
 ) ;
@@ -367,8 +278,15 @@ void amd_l2 (
 /* amd_valid */
 /* ------------------------------------------------------------------------- */
 
-/* Returns TRUE (1) if the matrix is valid as input to amd_order, FALSE (0)
- * otherwise.  For amd_order, the matrix must also be square. */
+/* Returns AMD_OK or AMD_OK_BUT_JUMBLED if the matrix is valid as input to
+ * amd_order; the latter is returned if the matrix has unsorted and/or
+ * duplicate row indices in one or more columns.  Returns AMD_INVALID if the
+ * matrix cannot be passed to amd_order.  For amd_order, the matrix must also
+ * be square.  The first two arguments are the number of rows and the number
+ * of columns of the matrix.  For its use in AMD, these must both equal n.
+ *
+ * NOTE: this routine returned TRUE/FALSE in v1.2 and earlier.
+ */
 
 int amd_valid
 (
@@ -378,12 +296,12 @@ int amd_valid
     const int Ai [ ]	    /* row indices, of size Ap [n_col] */
 ) ;
 
-long amd_l_valid
+UF_long amd_l_valid
 (
-    long n_row,
-    long n_col,
-    const long Ap [ ],
-    const long Ai [ ]
+    UF_long n_row,
+    UF_long n_col,
+    const UF_long Ap [ ],
+    const UF_long Ai [ ]
 ) ;
 
 /* ------------------------------------------------------------------------- */
@@ -393,11 +311,15 @@ long amd_l_valid
 /* The user can redefine these to change the malloc, free, and printf routines
  * that AMD uses. */
 
-extern void *(*amd_malloc) (size_t) ;		    /* pointer to malloc */
-extern void (*amd_free) (void *) ;		    /* pointer to free */
-extern void *(*amd_realloc) (void *, size_t) ;	    /* pointer to realloc */
-extern void *(*amd_calloc) (size_t, size_t) ;	    /* pointer to calloc */
-extern int (*amd_printf) (const char *, ...) ;	    /* pointer to printf */
+#ifndef EXTERN
+#define EXTERN extern
+#endif
+
+EXTERN void *(*amd_malloc) (size_t) ;		    /* pointer to malloc */
+EXTERN void (*amd_free) (void *) ;		    /* pointer to free */
+EXTERN void *(*amd_realloc) (void *, size_t) ;	    /* pointer to realloc */
+EXTERN void *(*amd_calloc) (size_t, size_t) ;	    /* pointer to calloc */
+EXTERN int (*amd_printf) (const char *, ...) ;	    /* pointer to printf */
 
 /* ------------------------------------------------------------------------- */
 /* AMD Control and Info arrays */
@@ -447,14 +369,18 @@ void amd_l_info     (double Info [ ]) ;
 /* ------------------------------------------------------------------------- */
 
 #define AMD_OK 0		/* success */
-#define AMD_OUT_OF_MEMORY -1	/* malloc failed */
+#define AMD_OUT_OF_MEMORY -1	/* malloc failed, or problem too large */
 #define AMD_INVALID -2		/* input arguments are not valid */
+#define AMD_OK_BUT_JUMBLED 1	/* input matrix is OK for amd_order, but
+    * columns were not sorted, and/or duplicate entries were present.  AMD had
+    * to do extra work before ordering the matrix.  This is a warning, not an
+    * error.  */
 
 /* ========================================================================== */
 /* === AMD version ========================================================== */
 /* ========================================================================== */
 
-/* AMD Version 1.2 and later will include the following definitions.
+/* AMD Version 1.2 and later include the following definitions.
  * As an example, to test if the version you are using is 1.2 or later:
  *
  * #ifdef AMD_VERSION
@@ -472,10 +398,10 @@ void amd_l_info     (double Info [ ]) ;
  * Versions 1.1 and earlier of AMD do not include a #define'd version number.
  */
 
-#define AMD_DATE "Aug. 30, 2005"
+#define AMD_DATE "May 5, 2006"
 #define AMD_VERSION_CODE(main,sub) ((main) * 1000 + (sub))
-#define AMD_MAIN_VERSION 1
-#define AMD_SUB_VERSION 2
+#define AMD_MAIN_VERSION 2
+#define AMD_SUB_VERSION 0
 #define AMD_VERSION AMD_VERSION_CODE(AMD_MAIN_VERSION,AMD_SUB_VERSION)
 
 #ifdef __cplusplus
